@@ -2,7 +2,7 @@
 # @Author: gviejo
 # @Date:   2022-04-04 22:35:44
 # @Last Modified by:   gviejo
-# @Last Modified time: 2022-04-04 22:41:37
+# @Last Modified time: 2022-08-19 09:03:33
 
 """Tests of neurosuite loader for `pynapple` package."""
 
@@ -12,17 +12,25 @@ import pandas as pd
 import pytest
 import warnings
 
+
 @pytest.mark.filterwarnings("ignore")
-def test_load_session():    
-    data = nap.load_session('nwbfilestest/neurosuite', 'neurosuite')
-    
+def test_load_session():
+    try:
+        data = nap.load_session("nwbfilestest/neurosuite", "neurosuite")
+    except:
+        data = nap.load_session("tests/nwbfilestest/neurosuite", "neurosuite")
+
+
 with warnings.catch_warnings():
     warnings.simplefilter("ignore")
-    data = nap.load_session('nwbfilestest/neurosuite', 'neurosuite')
+    try:
+        data = nap.load_session("nwbfilestest/neurosuite", "neurosuite")
+    except:
+        data = nap.load_session("tests/nwbfilestest/neurosuite", "neurosuite")
+
 
 @pytest.mark.parametrize("data", [data])
 class Test_Neurosuite:
-
     def test_epochs(self, data):
         epochs = data.epochs
         assert isinstance(epochs, dict)
@@ -42,8 +50,7 @@ class Test_Neurosuite:
         assert isinstance(data.time_support, nap.IntervalSet)
 
     def test_spikes(self, data):
-    	assert isinstance(data.spikes, nap.TsGroup)
-    	assert len(data.spikes) == 15
-    	for i in data.spikes.keys():
-    		assert len(data.spikes[i])
-
+        assert isinstance(data.spikes, nap.TsGroup)
+        assert len(data.spikes) == 15
+        for i in data.spikes.keys():
+            assert len(data.spikes[i])
